@@ -1,13 +1,17 @@
 package oviewer
 
-import "github.com/gdamore/tcell/v2"
+import (
+	"context"
 
-// setSectionDelimiterMode sets the inputMode to SectionDelimiter.
-func (root *Root) setSectionDelimiterMode() {
+	"github.com/gdamore/tcell/v2"
+)
+
+// inputSectionDelimiter sets the inputMode to SectionDelimiter.
+func (root *Root) inputSectionDelimiter(context.Context) {
 	input := root.input
 	input.reset()
-	input.SectionDelmCandidate.toLast(root.Doc.SectionDelimiter)
-	input.Event = newSectionDelimiterEvent(input.SectionDelmCandidate)
+	input.Candidate[SectionDelimiter].toLast(root.Doc.SectionDelimiter)
+	input.Event = newSectionDelimiterEvent(input.Candidate[SectionDelimiter])
 }
 
 // sectionDelimiterCandidate returns the candidate to set to default.
