@@ -10,18 +10,27 @@ import (
 	"os/signal"
 	"path/filepath"
 	"regexp"
+	"sort"
 	"sync"
 	"syscall"
 
-	"code.rocketnine.space/tslocum/cbind"
+	"codeberg.org/tslocum/cbind"
 	"github.com/fsnotify/fsnotify"
 	"github.com/gdamore/tcell/v2"
+	"github.com/noborus/tcellansi"
+	"github.com/spf13/viper"
 )
 
 // Root is the root structure of the oviewer.
 type Root struct {
 	// tcell.Screen is the root screen.
 	tcell.Screen
+
+	// settings contains the runtime settings template.
+	// These settings serve as the base configuration for each document.
+	// Individual documents can override these settings as needed.
+	settings RunTimeSettings
+
 	// Doc is the current document.
 	Doc *Document
 	// helpDoc is the help document.
@@ -42,6 +51,11 @@ type Root struct {
 	// inputKeyConfig contains the binding settings for the key.
 	inputKeyConfig *cbind.Configuration
 
+	// Pattern is the search pattern.
+	Pattern string
+	// OnExit is output on exit.
+	OnExit []string
+
 	// Original string.
 	OriginStr string
 
@@ -52,7 +66,6 @@ type Root struct {
 	message string
 	// cancelKeys represents the cancellation key string.
 	cancelKeys []string
-
 	// DocList is the list of documents.
 	DocList []*Document
 	// scr contains the screen information.
@@ -73,6 +86,8 @@ type Root struct {
 	// mu controls the RWMutex.
 	mu sync.RWMutex
 
+	// FollowAll is a follow mode for all documents.
+	FollowAll bool
 	// skipDraw is set to true when the mouse cursor just moves (no event occurs).
 	skipDraw bool
 }
@@ -130,14 +145,18 @@ func newLineNumber(number, wrap int) LineNumber {
 	return LineNumber{number: number, wrap: wrap}
 }
 
+// MinStartX is the minimum start position of x.
 const MinStartX = -10
 
 // RulerType is the type of ruler.
 type RulerType int
 
 const (
+	// RulerNone is no ruler.
 	RulerNone RulerType = iota
+	// RulerRelative is a relative ruler.
 	RulerRelative
+	// RulerAbsolute is an absolute ruler.
 	RulerAbsolute
 )
 
@@ -153,9 +172,9 @@ func (r RulerType) String() string {
 	}
 }
 
-// general structure contains the general of the display.
-// general contains values that determine the behavior of each document.
-type general struct {
+// RunTimeSettings structure contains the RunTimeSettings of the display.
+// RunTimeSettings contains values that determine the behavior of each document.
+type RunTimeSettings struct {
 	// Converter is the converter name.
 	Converter string
 	// Caption is an additional caption to display after the file name.
@@ -224,6 +243,48 @@ type general struct {
 	SectionHeader bool
 	// HideOtherSection is whether to hide other sections.
 	HideOtherSection bool
+
+	// Style is the style of the document.
+	Style Style
+}
+
+// Style structure contains the style settings of the display.
+type Style struct {
+	// ColumnRainbow is the style that applies to the column rainbow color highlight.
+	ColumnRainbow []OVStyle
+	// MultiColorHighlight is the style that applies to the multi color highlight.
+	MultiColorHighlight []OVStyle
+	// Header is the style that applies to the header.
+	Header OVStyle
+	// Body is the style that applies to the body.
+	Body OVStyle
+	// LineNumber is a style that applies line number.
+	LineNumber OVStyle
+	// SearchHighlight is the style that applies to the search highlight.
+	SearchHighlight OVStyle
+	// ColumnHighlight is the style that applies to the column highlight.
+	ColumnHighlight OVStyle
+	// MarkLine is a style that marked line.
+	MarkLine OVStyle
+	// SectionLine is a style that section delimiter line.
+	SectionLine OVStyle
+	// VerticalHeader is a style that applies to the vertical header.
+	VerticalHeader OVStyle
+	// JumpTargetLine is the line that displays the search results.
+	JumpTargetLine OVStyle
+	// Alternate is a style that applies line by line.
+	Alternate OVStyle
+	// Ruler is a style that applies to the ruler.
+	Ruler OVStyle
+	// HeaderBorder is the style that applies to the boundary line of the header.
+	// The boundary line of the header refers to the visual separator between the header and the rest of the content.
+	HeaderBorder OVStyle
+	// SectionHeaderBorder is the style that applies to the boundary line of the section header.
+	// The boundary line of the section header is the line that separates different sections in the header.
+	SectionHeaderBorder OVStyle
+	// VerticalHeaderBorder is the style that applies to the boundary character of the vertical header.
+	// The boundary character of the vertical header refers to the visual separator that delineates the vertical header from the rest of the content.
+	VerticalHeaderBorder OVStyle
 }
 
 var (
@@ -347,14 +408,15 @@ func NewOviewer(docs ...*Document) (*Root, error) {
 		return nil, ErrNotFound
 	}
 	root := &Root{
-		minStartX: MinStartX,
+		minStartX:      MinStartX,
+		settings:       NewRunTimeSettings(),
+		Config:         NewConfig(),
+		keyConfig:      cbind.NewConfiguration(),
+		inputKeyConfig: cbind.NewConfiguration(),
+		input:          NewInput(),
 	}
-	root.Config = NewConfig()
-	root.keyConfig = cbind.NewConfiguration()
-	root.inputKeyConfig = cbind.NewConfiguration()
 	root.DocList = append(root.DocList, docs...)
 	root.Doc = root.DocList[0]
-	root.input = NewInput()
 
 	screen, err := tcellNewScreen()
 	if err != nil {
@@ -374,6 +436,73 @@ func NewOviewer(docs ...*Document) (*Root, error) {
 	return root, nil
 }
 
+// NewRunTimeSettings returns the structure of RunTimeSettings with default values.
+func NewRunTimeSettings() RunTimeSettings {
+	return RunTimeSettings{
+		TabWidth:       8,
+		MarkStyleWidth: 1,
+		Converter:      convEscaped,
+		Style:          NewStyle(),
+	}
+}
+
+// NewStyle returns the structure of Style with default values.
+func NewStyle() Style {
+	return Style{
+		Header: OVStyle{
+			Bold: true,
+		},
+		Alternate: OVStyle{
+			Background: "gray",
+		},
+		LineNumber: OVStyle{
+			Bold: true,
+		},
+		SearchHighlight: OVStyle{
+			Reverse: true,
+		},
+		ColumnHighlight: OVStyle{
+			Reverse: true,
+		},
+		MarkLine: OVStyle{
+			Background: "darkgoldenrod",
+		},
+		SectionLine: OVStyle{
+			Background: "slateblue",
+		},
+		VerticalHeader: OVStyle{},
+		VerticalHeaderBorder: OVStyle{
+			Background: "#c0c0c0",
+		},
+		MultiColorHighlight: []OVStyle{
+			{Foreground: "red"},
+			{Foreground: "aqua"},
+			{Foreground: "yellow"},
+			{Foreground: "fuchsia"},
+			{Foreground: "lime"},
+			{Foreground: "blue"},
+			{Foreground: "grey"},
+		},
+		ColumnRainbow: []OVStyle{
+			{Foreground: "white"},
+			{Foreground: "crimson"},
+			{Foreground: "aqua"},
+			{Foreground: "lightsalmon"},
+			{Foreground: "lime"},
+			{Foreground: "blue"},
+			{Foreground: "yellowgreen"},
+		},
+		JumpTargetLine: OVStyle{
+			Underline: true,
+		},
+		Ruler: OVStyle{
+			Background: "#333333",
+			Foreground: "#CCCCCC",
+			Bold:       true,
+		},
+	}
+}
+
 // NewRoot returns the structure of the oviewer.
 // NewRoot is a simplified version that can be used externally.
 func NewRoot(r io.Reader) (*Root, error) {
@@ -465,9 +594,38 @@ func openFiles(fileNames []string) (*Root, error) {
 
 // SetConfig sets config.
 func (root *Root) SetConfig(config Config) {
-	viewMode, overwrite := config.Mode[config.ViewMode]
-	if overwrite {
-		config.General = mergeGeneral(config.General, viewMode)
+	// Old Style* settings are loaded with lower priority.
+	root.settings = setOldStyle(root.settings, config)
+	// General settings.
+	root.settings = updateRunTimeSettings(root.settings, config.General)
+
+	// view mode.
+	if config.ViewMode != "" {
+		viewMode, overwrite := config.Mode[config.ViewMode]
+		if overwrite {
+			root.settings = updateRunTimeSettings(root.settings, viewMode)
+		} else {
+			root.setMessageLogf("view mode not found: %s", config.ViewMode)
+		}
+	}
+
+	// Set the follow mode for all documents.
+	root.FollowAll = root.settings.FollowAll
+	// Set the minimum start position of x.
+	root.minStartX = config.MinStartX
+	// Set the caption from the environment variable.
+	if root.settings.Caption == "" {
+		root.settings.Caption = viper.GetString("CAPTION")
+	}
+
+	// Actually tabs when "\t" is specified as an option.
+	if root.settings.ColumnDelimiter == "\\t" {
+		root.settings.ColumnDelimiter = "\t"
+	}
+
+	// SectionHeader is enabled if SectionHeaderNum is greater than 0.
+	if root.settings.SectionHeaderNum > 0 {
+		root.settings.SectionHeader = true
 	}
 	root.Config = config
 }
@@ -543,7 +701,7 @@ func (root *Root) setKeyConfig(ctx context.Context) (map[string][]string, error)
 
 	keys, ok := keyBind[actionCancel]
 	if !ok {
-		log.Printf("no cancel key")
+		log.Println("no cancel key")
 	} else {
 		root.cancelKeys = keys
 	}
@@ -575,7 +733,7 @@ func (root *Root) Run() error {
 
 	// Quit if fits on screen.
 	if root.QuitSmall && root.DocumentLen() == 1 && root.docSmall() {
-		root.IsWriteOriginal = true
+		root.IsWriteOnExit = true
 		return nil
 	}
 
@@ -645,8 +803,8 @@ func (root *Root) prepareRun(ctx context.Context) error {
 // setCaption sets the caption.
 // optimizes execution with the Man command.
 func (root *Root) setCaption() {
-	if root.General.Caption != "" {
-		root.Doc.Caption = root.General.Caption
+	if root.settings.Caption != "" {
+		root.Doc.Caption = root.settings.Caption
 		return
 	}
 
@@ -661,18 +819,14 @@ func (root *Root) setCaption() {
 
 // setViewModeConfig sets view mode config.
 func (root *Root) setViewModeConfig() {
-	list := make([]string, 0, len(root.Config.Mode)+1)
-	list = append(list, nameGeneral)
-	for name := range root.Config.Mode {
-		list = append(list, name)
-	}
-	root.input.ModeCandidate.list = list
+	list := ListViewMode(root.Config)
+	root.input.Candidate[ViewMode].list = list
 }
 
 // prepareAllDocuments prepares all documents.
 func (root *Root) prepareAllDocuments() {
 	for n, doc := range root.DocList {
-		doc.general = root.Config.General
+		doc.RunTimeSettings = root.settings
 		doc.regexpCompile()
 
 		if doc.FollowName {
@@ -681,16 +835,18 @@ func (root *Root) prepareAllDocuments() {
 		if doc.ColumnWidth {
 			doc.ColumnMode = true
 		}
-		if doc.general.Converter != "" {
-			doc.conv = doc.converterType(doc.general.Converter)
+		if doc.RunTimeSettings.Converter != "" {
+			doc.conv = doc.converterType(doc.RunTimeSettings.Converter)
 		}
 		w := ""
-		if doc.general.WatchInterval > 0 {
+		if doc.RunTimeSettings.WatchInterval > 0 {
 			doc.watchMode()
 			w = "(watch)"
 		}
-		log.Printf("open [%d]%s%s", n, doc.FileName, w)
+		log.Printf("open [%d]%s%s\n", n, doc.FileName, w)
 	}
+	root.helpDoc.Style = root.settings.Style
+	root.logDoc.Style = root.settings.Style
 }
 
 // Close closes the oviewer.
@@ -744,100 +900,186 @@ func (root *Root) debugMessage(msg string) {
 	if len(msg) == 0 {
 		return
 	}
-	log.Printf("%s:%s", root.Doc.FileName, msg)
+	log.Printf("%s:%s\n", root.Doc.FileName, msg)
 }
 
-// mergeGeneral overwrites a general structure with a struct.
-func mergeGeneral(src general, dst general) general {
-	if dst.Converter != "" {
-		src.Converter = dst.Converter
+// setOldStyle applies deprecated style settings for backward compatibility.
+//
+// Deprecated: This function is planned to be removed in future versions.
+// It reads and applies old style settings to maintain compatibility with older configurations.
+// Use the new style configuration methods instead.
+func setOldStyle(src RunTimeSettings, config Config) RunTimeSettings {
+	src.Style.Body = config.StyleBody
+	src.Style.Header = config.StyleHeader
+	src.Style.LineNumber = config.StyleLineNumber
+	src.Style.SearchHighlight = config.StyleSearchHighlight
+	src.Style.ColumnHighlight = config.StyleColumnHighlight
+	src.Style.MarkLine = config.StyleMarkLine
+	src.Style.SectionLine = config.StyleSectionLine
+	src.Style.VerticalHeader = config.StyleVerticalHeader
+	src.Style.JumpTargetLine = config.StyleJumpTargetLine
+	src.Style.Alternate = config.StyleAlternate
+	src.Style.Ruler = config.StyleRuler
+	src.Style.HeaderBorder = config.StyleHeaderBorder
+	src.Style.SectionHeaderBorder = config.StyleSectionHeaderBorder
+	src.Style.VerticalHeaderBorder = config.StyleVerticalHeaderBorder
+	return src
+}
+
+// updateRunTimeSettings updates the RunTimeSettings.
+func updateRunTimeSettings(src RunTimeSettings, dst General) RunTimeSettings {
+	if dst.TabWidth != nil {
+		src.TabWidth = *dst.TabWidth
 	}
-	if dst.ColumnDelimiter != "" {
-		src.ColumnDelimiter = dst.ColumnDelimiter
+	if dst.Header != nil {
+		src.Header = *dst.Header
 	}
-	if dst.SectionDelimiter != "" {
-		src.SectionDelimiter = dst.SectionDelimiter
+	if dst.VerticalHeader != nil {
+		src.VerticalHeader = *dst.VerticalHeader
 	}
-	if dst.JumpTarget != "" {
-		src.JumpTarget = dst.JumpTarget
+	if dst.HeaderColumn != nil {
+		src.HeaderColumn = *dst.HeaderColumn
 	}
-	if len(dst.MultiColorWords) > 0 {
-		src.MultiColorWords = dst.MultiColorWords
+	if dst.SkipLines != nil {
+		src.SkipLines = *dst.SkipLines
 	}
-	if dst.TabWidth != 0 {
-		src.TabWidth = dst.TabWidth
+	if dst.WatchInterval != nil {
+		src.WatchInterval = *dst.WatchInterval
 	}
-	if dst.Header != 0 {
-		src.Header = dst.Header
+	if dst.MarkStyleWidth != nil {
+		src.MarkStyleWidth = *dst.MarkStyleWidth
 	}
-	if dst.VerticalHeader != 0 {
-		src.VerticalHeader = dst.VerticalHeader
+	if dst.SectionStartPosition != nil {
+		src.SectionStartPosition = *dst.SectionStartPosition
 	}
-	if dst.HeaderColumn != 0 {
-		src.HeaderColumn = dst.HeaderColumn
+	if dst.SectionHeaderNum != nil {
+		src.SectionHeaderNum = *dst.SectionHeaderNum
 	}
-	if dst.SkipLines != 0 {
-		src.SkipLines = dst.SkipLines
+	if dst.HScrollWidth != nil {
+		src.HScrollWidth = *dst.HScrollWidth
 	}
-	if dst.WatchInterval != 0 {
-		src.WatchInterval = dst.WatchInterval
+	if dst.HScrollWidthNum != nil {
+		src.HScrollWidthNum = *dst.HScrollWidthNum
 	}
-	if dst.MarkStyleWidth != 0 {
-		src.MarkStyleWidth = dst.MarkStyleWidth
+	if dst.RulerType != nil {
+		src.RulerType = *dst.RulerType
 	}
-	if dst.SectionStartPosition != 0 {
-		src.SectionStartPosition = dst.SectionStartPosition
+	if dst.AlternateRows != nil {
+		src.AlternateRows = *dst.AlternateRows
 	}
-	if dst.SectionHeaderNum != 0 {
-		src.SectionHeaderNum = dst.SectionHeaderNum
+	if dst.ColumnMode != nil {
+		src.ColumnMode = *dst.ColumnMode
 	}
-	if dst.HScrollWidth != "" {
-		src.HScrollWidth = dst.HScrollWidth
+	if dst.ColumnWidth != nil {
+		src.ColumnWidth = *dst.ColumnWidth
 	}
-	if dst.HScrollWidthNum != 0 {
-		src.HScrollWidthNum = dst.HScrollWidthNum
+	if dst.ColumnRainbow != nil {
+		src.ColumnRainbow = *dst.ColumnRainbow
 	}
-	if dst.RulerType != RulerNone {
-		src.RulerType = dst.RulerType
+	if dst.LineNumMode != nil {
+		src.LineNumMode = *dst.LineNumMode
 	}
-	if dst.AlternateRows {
-		src.AlternateRows = dst.AlternateRows
+	if dst.WrapMode != nil {
+		src.WrapMode = *dst.WrapMode
 	}
-	if dst.ColumnMode {
-		src.ColumnMode = dst.ColumnMode
+	if dst.FollowMode != nil {
+		src.FollowMode = *dst.FollowMode
 	}
-	if dst.ColumnWidth {
-		src.ColumnWidth = dst.ColumnWidth
+	if dst.FollowAll != nil {
+		src.FollowAll = *dst.FollowAll
 	}
-	if dst.ColumnRainbow {
-		src.ColumnRainbow = dst.ColumnRainbow
+	if dst.FollowSection != nil {
+		src.FollowSection = *dst.FollowSection
 	}
-	if dst.LineNumMode {
-		src.LineNumMode = dst.LineNumMode
+	if dst.FollowName != nil {
+		src.FollowName = *dst.FollowName
 	}
-	if !dst.WrapMode { // Because wrap mode defaults to true.
-		src.WrapMode = dst.WrapMode
+	if dst.PlainMode != nil {
+		src.PlainMode = *dst.PlainMode
 	}
-	if dst.FollowMode {
-		src.FollowMode = dst.FollowMode
+	if dst.SectionHeader != nil {
+		src.SectionHeader = *dst.SectionHeader
 	}
-	if dst.FollowAll {
-		src.FollowAll = dst.FollowAll
+	if dst.HideOtherSection != nil {
+		src.HideOtherSection = *dst.HideOtherSection
 	}
-	if dst.FollowSection {
-		src.FollowSection = dst.FollowSection
+	if dst.ColumnDelimiter != nil {
+		src.ColumnDelimiter = *dst.ColumnDelimiter
 	}
-	if dst.FollowName {
-		src.FollowName = dst.FollowName
+	if dst.SectionDelimiter != nil {
+		src.SectionDelimiter = *dst.SectionDelimiter
 	}
-	if dst.SectionHeader {
-		src.SectionHeader = dst.SectionHeader
+	if dst.JumpTarget != nil {
+		src.JumpTarget = *dst.JumpTarget
 	}
-	if dst.HideOtherSection {
-		src.HideOtherSection = dst.HideOtherSection
+	if dst.MultiColorWords != nil {
+		src.MultiColorWords = *dst.MultiColorWords
 	}
-	if dst.PlainMode {
-		src.PlainMode = dst.PlainMode
+	if dst.Caption != nil {
+		src.Caption = *dst.Caption
+	}
+	if dst.Converter != nil {
+		src.Converter = *dst.Converter
+	}
+	if dst.Align != nil && *dst.Align {
+		src.Converter = convAlign
+	}
+	if dst.Raw != nil && *dst.Raw {
+		src.Converter = convRaw
+	}
+	src.Style = updateRuntimeStyle(src.Style, dst.Style)
+	return src
+}
+
+// updateRuntimeStyle updates the style.
+func updateRuntimeStyle(src Style, dst StyleConfig) Style {
+	if dst.ColumnRainbow != nil {
+		src.ColumnRainbow = *dst.ColumnRainbow
+	}
+	if dst.MultiColorHighlight != nil {
+		src.MultiColorHighlight = *dst.MultiColorHighlight
+	}
+	if dst.Header != nil {
+		src.Header = *dst.Header
+	}
+	if dst.Body != nil {
+		src.Body = *dst.Body
+	}
+	if dst.LineNumber != nil {
+		src.LineNumber = *dst.LineNumber
+	}
+	if dst.SearchHighlight != nil {
+		src.SearchHighlight = *dst.SearchHighlight
+	}
+	if dst.ColumnHighlight != nil {
+		src.ColumnHighlight = *dst.ColumnHighlight
+	}
+	if dst.MarkLine != nil {
+		src.MarkLine = *dst.MarkLine
+	}
+	if dst.SectionLine != nil {
+		src.SectionLine = *dst.SectionLine
+	}
+	if dst.VerticalHeader != nil {
+		src.VerticalHeader = *dst.VerticalHeader
+	}
+	if dst.JumpTargetLine != nil {
+		src.JumpTargetLine = *dst.JumpTargetLine
+	}
+	if dst.Alternate != nil {
+		src.Alternate = *dst.Alternate
+	}
+	if dst.Ruler != nil {
+		src.Ruler = *dst.Ruler
+	}
+	if dst.HeaderBorder != nil {
+		src.HeaderBorder = *dst.HeaderBorder
+	}
+	if dst.SectionHeaderBorder != nil {
+		src.SectionHeaderBorder = *dst.SectionHeaderBorder
+	}
+	if dst.VerticalHeaderBorder != nil {
+		src.VerticalHeaderBorder = *dst.VerticalHeaderBorder
 	}
 	return src
 }
@@ -853,7 +1095,7 @@ func (root *Root) docSmall() bool {
 	for y := 0; y < m.BufEndNum(); y++ {
 		lc, err := m.contents(y)
 		if err != nil {
-			log.Printf("docSmall %d: %s", y, err)
+			log.Printf("docSmall %d: %v\n", y, err)
 			continue
 		}
 		height += 1 + (len(lc) / root.scr.vWidth)
@@ -864,12 +1106,103 @@ func (root *Root) docSmall() bool {
 	return true
 }
 
+// OutputOnExit outputs to the terminal when exiting.
+func (root *Root) OutputOnExit() {
+	output := os.Stdout
+	root.outputOnExit(output)
+}
+
 // WriteOriginal writes to the original terminal.
 func (root *Root) WriteOriginal() {
 	output := os.Stdout
 	root.writeOriginal(output)
 }
 
+// WriteCurrentScreen writes to the current screen.
+func (root *Root) WriteCurrentScreen() {
+	output := os.Stdout
+	root.writeCurrentScreen(output)
+}
+
+// outputOnExit outputs to the terminal when exiting.
+func (root *Root) outputOnExit(output io.Writer) {
+	if root.IsWriteOnExit {
+		if root.IsWriteOriginal {
+			root.writeOriginal(output)
+		} else {
+			root.writeCurrentScreen(output)
+		}
+	}
+	if root.Debug {
+		root.writeLog(output)
+	}
+}
+
+// writeCurrentScreen writes to the current screen.
+func (root *Root) writeCurrentScreen(output io.Writer) {
+	strs := root.OnExit
+	if strs == nil {
+		height, err := root.dummyScreen()
+		if err != nil {
+			log.Println(err)
+			return
+		}
+		strs = tcellansi.ScreenContentToStrings(root.Screen, 0, root.Doc.width, 0, height-1)
+	}
+	for _, str := range strs {
+		if _, err := output.Write([]byte(str)); err != nil {
+			log.Println(err)
+			return
+		}
+	}
+}
+
+// dummyScreen creates a dummy screen.
+func (root *Root) dummyScreen() (int, error) {
+	root.Screen = tcell.NewSimulationScreen("")
+	if err := root.Screen.Init(); err != nil {
+		return 0, err
+	}
+	height := root.scr.vHeight
+	if root.BeforeWriteOriginal != 0 || root.AfterWriteOriginal != 0 {
+		root.Doc.topLN = max(root.Doc.topLN+root.Doc.firstLine()-root.BeforeWriteOriginal, 0)
+		end := root.Doc.bottomLN
+		if root.AfterWriteOriginal != 0 {
+			end = root.Doc.topLN + root.AfterWriteOriginal
+		}
+		height = end - root.Doc.topLN
+	}
+	root.Screen.SetSize(root.scr.vWidth, height)
+	if root.Pattern != "" {
+		root.setSearcher(root.Pattern, root.Config.CaseSensitive)
+	}
+	ctx := context.Background()
+	root.prepareScreen()
+	root.prepareDraw(ctx)
+	root.draw(ctx)
+	height = realHeight(root.scr) + 1
+	return height, nil
+}
+
+func realHeight(scr SCR) int {
+	height := 0
+	for ; height < scr.vHeight-1; height++ {
+		if !scr.lines[scr.numbers[height].number].valid {
+			break
+		}
+	}
+	return height
+}
+
+// ScreenContent returns the screen content.
+func (root *Root) ScreenContent() []string {
+	root.Screen.Sync()
+	m := root.Doc
+	height := realHeight(root.scr)
+	return tcellansi.ScreenContentToStrings(root.Screen, 0, m.width, 0, height)
+}
+
+// writeOriginal writes to the original terminal.
 func (root *Root) writeOriginal(output io.Writer) {
 	m := root.Doc
 	if m.bottomLN == 0 {
@@ -911,11 +1244,11 @@ func (root *Root) WriteLog() {
 	root.writeLog(os.Stdout)
 }
 
-func (root *Root) writeLog(w io.Writer) {
+func (root *Root) writeLog(output io.Writer) {
 	m := root.logDoc
 	start := max(0, m.BufEndNum()-MaxWriteLog)
 	end := m.BufEndNum()
-	if err := m.Export(w, start, end); err != nil {
+	if err := m.Export(output, start, end); err != nil {
 		log.Println(err)
 	}
 }
@@ -941,17 +1274,28 @@ func (root *Root) debugNumOfChunk() {
 	for _, doc := range root.DocList {
 		if !doc.seekable {
 			if MemoryLimit > 0 {
-				log.Printf("%s: The number of chunks is %d, of which %d(%v) are loaded", doc.FileName, len(doc.store.chunks), doc.store.loadedChunks.Len(), doc.store.loadedChunks.Keys())
+				log.Printf("%s: The number of chunks is %d, of which %d(%v) are loaded\n", doc.FileName, len(doc.store.chunks), doc.store.loadedChunks.Len(), doc.store.loadedChunks.Keys())
 			}
 			continue
 		}
 		for n, chunk := range doc.store.chunks {
 			if n != 0 && len(chunk.lines) != 0 {
 				if !doc.store.loadedChunks.Contains(n) {
-					log.Printf("chunk %d is not under control %d", n, len(chunk.lines))
+					log.Printf("chunk %d is not under control %d\n", n, len(chunk.lines))
 				}
 			}
 		}
-		log.Printf("%s(seekable): The number of chunks is %d, of which %d(%v) are loaded", doc.FileName, len(doc.store.chunks), doc.store.loadedChunks.Len(), doc.store.loadedChunks.Keys())
+		log.Printf("%s(seekable): The number of chunks is %d, of which %d(%v) are loaded\n", doc.FileName, len(doc.store.chunks), doc.store.loadedChunks.Len(), doc.store.loadedChunks.Keys())
 	}
 }
+
+// ListViewMode returns the list of view modes.
+func ListViewMode(config Config) []string {
+	list := make([]string, 0, len(config.Mode)+1)
+	list = append(list, nameGeneral)
+	for name := range config.Mode {
+		list = append(list, name)
+	}
+	sort.Strings(list)
+	return list
+}
